import karamba
import os
import re

def isOnLine(stateDir) :
	f = open(stateDir + "/state","r")
	ac = f.readline()
	f.close()
	acmatch = re.search("on-line", ac)
	if ( acmatch != None ) :
		return True
	else :
		return False

def getAcpiInfos():
	f = os.popen("/usr/bin/acpi")
	batt = f.readline()
	f.close()
	match = re.search(": ([a-z]+), ([0-9]+)%(, )?(([0-9]+):([0-9]+):[0-9]+)?",batt)
	load = int(match.group(2))
	if ( match.group(1) == "charged" ) :
		return (100, "charged")
	else :
		time = str(int(match.group(5))) +" h " + match.group(6) + " min"
		return (load, time)

class batteryModule:
	
	def __init__(self, widget, theme, x, y):
		self.theme = theme
		self.mainImg = karamba.createImage(widget, x, y, "")
		self.bottomImg = karamba.createImage(widget, x, y + 91, "Themes/" + theme + "/acpi/battery_bottom.png")
		self.bar = karamba.createBar(widget, x, y+17, 74, 74, "Themes/" + theme + "/acpi/battery_bar.png")
		self.topImg = karamba.createImage(widget, x + 9, y, "Themes/" + theme + "/acpi/battery_top.png")
		self.maskImg = karamba.createImage(widget, x, y, "Themes/" + theme + "/acpi/battery_mask.png")
		self.loadText = karamba.createText(widget, x, y + 43, 74, 22, "")
		karamba.changeTextSize(widget, self.loadText, 20)
		self.x = x
		self.y = y
		
		karamba.hideImage(widget, self.bottomImg)
		karamba.hideImage(widget, self.topImg)
		karamba.hideImage(widget, self.maskImg)
		karamba.hideBar(widget, self.bar)
		karamba.setBarVertical(widget, self.bar, 1)
		karamba.setTextAlign(widget, self.loadText, "CENTER")
	
	def update(self, widget, isOnAc, load):
		text_x = karamba.getTextPos(widget, self.loadText)[0]
		karamba.changeText(widget, self.loadText, str(load) + "%")
		if isOnAc:
			if ( text_x != self.x + 74 ):
				karamba.moveText(widget, self.loadText, self.x + 78, self.y + 43)
			karamba.setImagePath(widget, self.mainImg, "Themes/" + self.theme + "/acpi/externalpower.png")
			karamba.hideImage(widget, self.bottomImg)
			karamba.hideImage(widget, self.topImg)
			karamba.hideImage(widget, self.maskImg)
			karamba.hideBar(widget, self.bar)
		else:
			if ( text_x != self.x + 74 ):
				karamba.moveText(widget, self.loadText, self.x, self.y + 43)
			if ( load == 0 ):
				karamba.setImagePath(widget, self.mainImg, "Themes/" + self.theme + "/acpi/battery_empty.png")
				karamba.hideImage(widget, self.bottomImg)
				karamba.hideImage(widget, self.topImg)
				karamba.hideImage(widget, self.maskImg)
				karamba.hideBar(widget, self.bar)
			elif ( load == 100):
				karamba.setImagePath(widget, self.mainImg, "Themes/" + self.theme + "/acpi/battery_full.png")
				karamba.hideImage(widget, self.bottomImg)
				karamba.hideImage(widget, self.topImg)
				karamba.hideImage(widget, self.maskImg)
				karamba.hideBar(widget, self.bar)
			else:
				top_y = int((float(100 - load) / 100.0) * 74.0) + 10 + self.y
				karamba.setImagePath(widget, self.mainImg, "Themes/" + self.theme + "/acpi/battery_bg.png")
				karamba.showImage(widget, self.bottomImg)
				karamba.showImage(widget, self.topImg)
				karamba.showImage(widget, self.maskImg)
				karamba.showBar(widget, self.bar)
				karamba.setBarValue(widget, self.bar, load)
				karamba.moveImage(widget, self.topImg, self.x + 9, top_y)
	
	def close(self, widget):
		karamba.deleteImage(widget, self.mainImg)
		karamba.deleteImage(widget, self.bottomImg)
		karamba.deleteImage(widget, self.topImg)
		karamba.deleteImage(widget, self.maskImg)
		karamba.deleteBar(widget, self.bar)
		karamba.deleteText(widget, self.loadText)

class AcpiInfos:
	
	def __init__(self, widget, theme, color, titleColor, stateDir, inter):
		self.loaded = False
		self.count = inter
		self.interval = inter
		self.title = karamba.createText(widget, 36, 32, 200, 24, "Battery")
		karamba.changeTextSize(widget, self.title, 20)
		karamba.changeTextColor(widget, self.title, titleColor[0], titleColor[1], titleColor[2])
		self.line = karamba.createImage(widget, 31, 58, "Themes/" + theme + "/line.png")
		
		self.stateDir = stateDir
		self.timeText = karamba.createText(widget, 34, 114, 113, 12, "")
		karamba.setTextAlign(widget, self.timeText, "CENTER")
		self.batModule = batteryModule(widget, theme, 143, 70)
		
		karamba.changeTextColor(widget, self.batModule.loadText, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.timeText, color[0], color[1], color[2])
		self.loaded = True
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.batModule.loadText, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.timeText, color[0], color[1], color[2])
	
	def shadow(self, widget, bool):
		if bool:
			karamba.changeTextShadow(widget, self.batModule.loadText, 1)
			karamba.changeTextShadow(widget, self.timeText, 1)
		else:
			karamba.changeTextShadow(widget, self.batModule.loadText, 0)
			karamba.changeTextShadow(widget, self.timeText, 0)
	
	def update(self, widget):
		if ( self.count < self.interval ):
			self.count = self.count + 1
		else:
			self.count = 1
			infos = getAcpiInfos()
			status = isOnLine(self.stateDir)
			self.batModule.update(widget, status, infos[0])
			if status:
				text = infos[1]
				if ( text == "charged" ):
					karamba.changeText(widget, self.timeText, "charged")
				else:
					karamba.changeText(widget, self.timeText, text + "\nuntil charged")
			else:
				karamba.changeText(widget, self.timeText, infos[1] + "\nremaining")
	
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		karamba.deleteText(widget, self.timeText)
		self.batModule.close(widget)