############################################
#                                                 Aero AIO                                                    #
#                     By Adrien Vermeulen - Released under GPL                         #
############################################

 #this import statement allows access to the karamba functions
import karamba
import re
import sysinfos_aio, cpu_mem_aio, partitions_aio, network_aio, clock_aio, calendar_aio, mail_aio, acpi_aio

infosImg, cpuImg, hddImg, netImg, clockImg, calImg, mailImg, battImg = None, None, None, None, None, None, None, None
conf_pid = None
page = None
autoswitch_count = 0
as_state = True
mode = True

conf = {}

def restoreConfig(widget):
	global conf
	print "****** Checking and restoring the configuration... ******"
	default_conf = { "color" : [89, 89, 89],
		"titleColor" : [89, 89, 89],
		"shadow" : False,
		"titleShadow" : False,
		"theme" : "default",
		"icon" : "default",
		"configuration" : "11111100",
		"temp_file" : "/proc/acpi/thermal_zone/THRM/temperature",
		"cpu_interval" : 5,
		"graphColor" : [89, 89, 89],
		"partition_list" : ['Root', '/', 'Home', '/home'],
		"net_list" : ['eth0', 512, 64],
		"net_interval" : 1200,
		"mail_list" : [],
		"mail_interval" : 1200,
		"batt_stateDir" : "/proc/acpi/ac_adapter/AC",
		"batt_interval" : 5,
		"autoswitch" : [60]}
	changed = False
		
	for key in default_conf:
		try:
			conf[key]
		except:
			print key + "..........: missing, using default value"
			conf[key] = default_conf[key]
			changed = True
		else:
			print key + "..........: OK"
	
	if changed:
		karamba.writeConfigEntry(widget, "conf", str(conf))
		print "****** Configuration checked and restored ******"
	else:
		print "****** Configuration checked ******"

def translateImg(widget, img, pos_a, pos_b, a, M):
	from math import sqrt
	if ( (pos_b[0] - pos_a[0]) != 0  or (pos_b[1] - pos_a[1]) != 0 ):
		if ( a < 0 ):
			ax = a * (float(pos_b[0] - pos_a[0]) / sqrt((pos_b[0] - pos_a[0])*(pos_b[0] - pos_a[0])+(pos_b[1] - pos_a[1])*(pos_b[1] - pos_a[1])))
			ay = a * (float(pos_b[1] - pos_a[1]) / sqrt((pos_b[0] - pos_a[0])*(pos_b[0] - pos_a[0])+(pos_b[1] - pos_a[1])*(pos_b[1] - pos_a[1])))
			Tr = sqrt( 2 * float(pos_b[0] - pos_a[0]) / -ax )
			T = int(Tr)
			Vx = -ax * T
			Vy = -ay * T
			dx = ((pos_b[0] - pos_a[0]) + int(-0.5 * ax * T * T - Vx * T))
			dy = ((pos_b[1] - pos_a[1]) + int(-0.5 * ay * T * T - Vy * T))
		elif ( a > 0 ):
			ax = a * (float(pos_b[0] - pos_a[0]) / sqrt((pos_b[0] - pos_a[0])*(pos_b[0] - pos_a[0])+(pos_b[1] - pos_a[1])*(pos_b[1] - pos_a[1])))
			ay = a * (float(pos_b[1] - pos_a[1]) / sqrt((pos_b[0] - pos_a[0])*(pos_b[0] - pos_a[0])+(pos_b[1] - pos_a[1])*(pos_b[1] - pos_a[1])))
			Tr = sqrt( 2 * float(pos_b[0] - pos_a[0]) / ax )
			T = int(Tr)
			dx = 0
			dy = 0
			Vx = 0.0
			Vy = 0.0
		elif ( a == 0 ):
			ax = 0.0
			ay = 0.0
			T = 24
			dx = 0
			dy = 0
			Vx = float(pos_b[0] - pos_a[0]) / T
			Vy = float(pos_b[1] - pos_a[1]) / T
		
		for i in range(M * T + 1):
			t = i / float(M)
			x = int( 0.5 * ax * t * t + Vx * t) + pos_a[0] + dx
			y = int( 0.5 * ay * t * t + Vy * t) + pos_a[1] + dy
			karamba.moveImage(widget, img, x, y)
			karamba.redrawWidget(widget)

def imgPressed(widget, meter):
	pos = karamba.getImagePos(widget, meter)
	dim = (karamba.getImageWidth(widget, meter), karamba.getImageHeight(widget, meter))
	for i in range(1,22,4):
		percent = (100.0 - i) / 100.0
		new_dim = int(dim[0] * percent), int(dim[1] * percent)
		new_pos = (dim[0] - new_dim[0]) / 2 + pos[0], (dim[1] - new_dim[1]) / 2 + pos[1]
		karamba.resizeImage(widget, meter, new_dim[0], new_dim[1])
		karamba.moveImage(widget, meter,  new_pos[0], new_pos[1])
		karamba.redrawWidget(widget)
	for i in range(1,22,4):
		percent = (79.0 + i) / 100.0
		new_dim = int(dim[0] * percent), int(dim[1] * percent)
		new_pos = (dim[0] - new_dim[0]) / 2 + pos[0], (dim[1] - new_dim[1]) / 2 + pos[1]
		karamba.resizeImage(widget, meter, new_dim[0], new_dim[1])
		karamba.moveImage(widget, meter,  new_pos[0], new_pos[1])
		karamba.redrawWidget(widget)

def loadIcons(widget):
	global infosImg, cpuImg, hddImg, netImg, clockImg, calImg, mailImg, battImg, conf
	num = 0
	configuration = conf["configuration"]
	icons = conf["icon"]
	page_list = []
	for i in configuration:
		num = num + int(i)
	space = (290 - num * 32) / (num + 1)
	rest = (290 - num * 32) % (num + 1)
	incr = space + 32
	x = 35 + space + rest / 2
	if ( configuration[0] == '1' ):
		page_list.append('SysInfos')
		infosImg = karamba.createImage(widget, x, 186, "icons/" + icons + "/pc.png")
		y = 186 + (32 - karamba.getImageHeight(widget, infosImg)) / 2
		x_real = x + (32 - karamba.getImageWidth(widget, infosImg)) / 2
		karamba.moveImage(widget, infosImg, x_real, y)
		karamba.attachClickArea(widget, infosImg)
		x = x + incr
	if ( configuration[1] == '1' ):
		page_list.append('CpuMem')
		cpuImg = karamba.createImage(widget, x, 186, "icons/" + icons + "/memory.png")
		y = 186 + (32 - karamba.getImageHeight(widget, cpuImg)) / 2
		x_real = x + (32 - karamba.getImageWidth(widget, cpuImg)) / 2
		karamba.moveImage(widget, cpuImg, x_real, y)
		karamba.attachClickArea(widget, cpuImg)
		x = x + incr
	if ( configuration[2] == '1' ):
		page_list.append('Partitions')
		hddImg = karamba.createImage(widget, x, 186, "icons/" + icons + "/hdd.png")
		y = 186 + (32 - karamba.getImageHeight(widget, hddImg)) / 2
		x_real = x + (32 - karamba.getImageWidth(widget, hddImg)) / 2
		karamba.moveImage(widget, hddImg, x_real, y)
		karamba.attachClickArea(widget, hddImg)
		x = x + incr
	if ( configuration[3] == '1' ):
		page_list.append('Network')
		netImg = karamba.createImage(widget, x, 186, "icons/" + icons + "/network.png")
		y = 186 + (32 - karamba.getImageHeight(widget, netImg)) / 2
		x_real = x + (32 - karamba.getImageWidth(widget, netImg)) / 2
		karamba.moveImage(widget, netImg, x_real, y)
		karamba.attachClickArea(widget, netImg)
		x = x + incr
	if ( configuration[4] == '1' ):
		page_list.append('Clock')
		clockImg = karamba.createImage(widget, x, 186, "icons/" + icons + "/clock.png")
		y = 186 + (32 - karamba.getImageHeight(widget, clockImg)) / 2
		x_real = x + (32 - karamba.getImageWidth(widget, clockImg)) / 2
		karamba.moveImage(widget, clockImg, x_real, y)
		karamba.attachClickArea(widget, clockImg)
		x = x + incr
	if ( configuration[5] == '1' ):
		page_list.append('Calendar')
		calImg = karamba.createImage(widget, x, 186, "icons/" + icons + "/calendar.png")
		y = 186 + (32 - karamba.getImageHeight(widget, calImg)) / 2
		x_real = x + (32 - karamba.getImageWidth(widget, calImg)) / 2
		karamba.moveImage(widget, calImg, x_real, y)
		karamba.attachClickArea(widget, calImg)
		x = x + incr
	if ( configuration[6] == '1' ):
		page_list.append('Mail')
		mailImg = karamba.createImage(widget, x, 186, "icons/" + icons + "/email.png")
		y = 186 + (32 - karamba.getImageHeight(widget, mailImg)) / 2
		x_real = x + (32 - karamba.getImageWidth(widget, mailImg)) / 2
		karamba.moveImage(widget, mailImg, x_real, y)
		karamba.attachClickArea(widget, mailImg)
		x = x + incr
	if ( configuration[7] == '1' ):
		page_list.append('AcpiInfos')
		battImg = karamba.createImage(widget, x, 186, "icons/" + icons + "/energy.png")
		y = 186 + (32 - karamba.getImageHeight(widget, battImg)) / 2
		x_real = x + (32 - karamba.getImageWidth(widget, battImg)) / 2
		karamba.moveImage(widget, battImg, x_real, y)
		karamba.attachClickArea(widget, battImg)
	return page_list

def closeIcons(widget):
	global infosImg, cpuImg, hddImg, netImg, clockImg, calImg, mailImg, battImg, conf
	
	configuration = conf["configuration"]
	if ( configuration[0] == '1' ):
		karamba.deleteImage(widget, infosImg)
	if ( configuration[1] == '1' ):
		karamba.deleteImage(widget, cpuImg)
	if ( configuration[2] == '1' ):
		karamba.deleteImage(widget, hddImg)
	if ( configuration[3] == '1' ):
		karamba.deleteImage(widget, netImg)
	if ( configuration[4] == '1' ):
		karamba.deleteImage(widget, clockImg)
	if ( configuration[5] == '1' ):
		karamba.deleteImage(widget, calImg)
	if ( configuration[6] == '1' ):
		karamba.deleteImage(widget, mailImg)
	if ( configuration[7] == '1' ):
		karamba.deleteImage(widget, battImg)

def loadPage(widget, name):
	global page, conf, mode, infosImg, cpuImg, hddImg, netImg, clockImg, calImg, mailImg, battImg, selectImg
	
	theme = conf["theme"]
	color = conf["color"]
	titleColor = conf["titleColor"]
	
	if ( name == 'SysInfos' ):
		page = sysinfos_aio.SysInfos(widget, theme, color, titleColor)
		meter = infosImg
	if ( name == 'CpuMem' ):
		page = cpu_mem_aio.CpuMem(widget, theme, color, titleColor, conf["graphColor"], conf["temp_file"], conf["cpu_interval"])
		meter = cpuImg
	if ( name == 'Partitions' ):
		page = partitions_aio.Partitions(widget, theme, color, titleColor, conf["partition_list"])
		meter = hddImg
	if ( name == 'Network' ):
		page = network_aio.Network(widget, theme, color, titleColor, conf["net_list"], conf["net_interval"])
		meter = netImg
	if ( name == 'Clock' ):
		page = clock_aio.Clock(widget, theme, color, titleColor, mode)
		meter = clockImg
	if ( name == 'Calendar' ):
		page = calendar_aio.Calendar(widget, theme, color, titleColor)
		meter = calImg
	if ( name == 'Mail' ):
		page = mail_aio.Mail(widget, theme, color, titleColor, conf["mail_list"], conf["mail_interval"])
		meter = mailImg
	if ( name == 'AcpiInfos' ):
		page = acpi_aio.AcpiInfos(widget, theme, color, titleColor, conf["batt_stateDir"], conf["batt_interval"])
		meter = battImg
	setShadow(widget)
	pos = karamba.getImagePos(widget, meter)
	posImg = karamba.getImagePos(widget, selectImg)
	dimImg = (karamba.getImageWidth(widget, selectImg), karamba.getImageHeight(widget, selectImg))
	dimMeter = (karamba.getImageWidth(widget, meter), karamba.getImageHeight(widget, meter))
	x = pos[0] + (dimMeter[0] - dimImg[0]) / 2
	y = pos[1] + (dimMeter[1] - dimImg[1]) / 2
	translateImg(widget, selectImg, posImg, (x, y), -3.5, 8)
	
	karamba.writeConfigEntry(widget, "last_page", name)

def updateTheme(widget):
	global conf, bg
	karamba.deleteImage(widget, bg)
	theme = conf["theme"]
	bg = karamba.createBackgroundImage(widget, 0, 0, "Themes/" + theme + "/bg.png")
	karamba.setImagePath(widget, autoswitchImg, "Themes/" + theme + "/autoswitch.png")
	karamba.setImagePath(widget, stateImg, "Themes/" + theme + "/play.png")
	karamba.setImagePath(widget, selectImg, "Themes/" + theme + "/selected.png")
	if ( len(conf["autoswitch"]) > 2 ):
		karamba.showImage(widget, autoswitchImg)
		karamba.showImage(widget, stateImg)
	else:
		karamba.hideImage(widget, autoswitchImg)
		karamba.hideImage(widget, stateImg)

def updateStateImg(widget, bool):
	global conf, as_state, stateImg
	if bool:
		karamba.setImagePath(widget, stateImg, "Themes/" + conf["theme"] + "/play.png")
	else:
		karamba.setImagePath(widget, stateImg, "Themes/" + conf["theme"] + "/pause.png")
	as_state = bool

def setShadow(widget):
	global page, conf
	page.shadow(widget, conf["shadow"])
	if conf["titleShadow"]:
		karamba.changeTextShadow(widget, page.title, 1)
	else:
		karamba.changeTextShadow(widget, page.title, 0)

#this is called when your widget is initialized
def initWidget(widget):
	global page, bg, autoswitchImg, as_state, stateImg, selectImg, conf, mode
	
	if ( karamba.readConfigEntry(widget, "conf") != None ):
		conf = eval(karamba.readConfigEntry(widget, "conf"))
	restoreConfig(widget)
	if ( karamba.readConfigEntry(widget, "mode") != None ):
		mode = eval(karamba.readConfigEntry(widget, "mode"))
	
	theme = conf["theme"]
	bg = karamba.createBackgroundImage(widget, 0, 0, "Themes/" + theme + "/bg.png")
	selectImg = karamba.createImage(widget, 30, 186, "Themes/" + theme + "/selected.png")
	loadIcons(widget)
	
	autoswitchImg = karamba.createImage(widget, 302, 36, "Themes/" + theme + "/autoswitch.png")
	stateImg = karamba.createImage(widget, 311, 43, "Themes/" + theme + "/play.png")
	karamba.attachClickArea(widget, autoswitchImg)
	if ( len(conf["autoswitch"]) > 2 ):
		karamba.showImage(widget, autoswitchImg)
		karamba.showImage(widget, stateImg)
	else:
		karamba.hideImage(widget, autoswitchImg)
		karamba.hideImage(widget, stateImg)
	
	if ( karamba.readConfigEntry(widget, "last_page") != None ):
		name = karamba.readConfigEntry(widget, "last_page")
		loadPage(widget, name)
	else:
		loadPage(widget, 'SysInfos')
	setShadow(widget)
	
	karamba.redrawWidget(widget)
	
	karamba.addMenuConfigOption(widget, "confclick", "Configure theme")
	karamba.setMenuConfigOption(widget, "confclick", 0)

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
	global page, autoswitch_count, as_state
	autoswitch = conf["autoswitch"]
	if ( len(autoswitch) > 2 and (autoswitch_count % autoswitch[0]) == 0):
		page.close(widget)
		page = None
		index = autoswitch_count / autoswitch[0]
		if ( index + 1 >= len(autoswitch) ):
			autoswitch_count = 0
			index = 0
		loadPage(widget, autoswitch[index + 1])
	if as_state:
		autoswitch_count = autoswitch_count + 1
	
	page.update(widget)
	#karamba.redrawWidget(widget)

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
	global conf_pid, conf
	
	if ( key == "confclick" ):
		karamba.setMenuConfigOption(widget, "confclick", 0)
		path = karamba.getThemePath(widget)
		arg = conf
		arg['theme_path'] = path
		conf_command = ["python", path + "scripts/config_dialog.py", str(conf)]
		conf_pid = karamba.executeInteractive(widget, conf_command)

def commandOutput(widget, pid, output):
	global color_pid,  custom_pid, conf_pid, page, configuration, icons, conf, as_state, autoswitch_count
	
	name = page.__class__.__name__
	if ( name == "Mail" ):
		for module in page.module_list:
			if ( pid == module.pid ):
				num = output.replace('\n','')
				karamba.changeText(widget, module.num, num)
				text = karamba.getTextValue(widget, module.name)
				karamba.changeText(widget, module.text, "new message(s)")
				print "** " + text + " updated **"
				karamba.redrawWidget(widget)
		karamba.redrawWidget(widget)
	
	if ( pid == conf_pid ):
		page.close(widget)
		page = None
		f = compile(output.replace('\n',''), '<string>', 'exec')
		exec f
		closeIcons(widget)
		conf = confr
		karamba.writeConfigEntry(widget, "conf", str(conf))
		
		page_list = loadIcons(widget)
		updateTheme(widget)
		as_state = True
		autoswitch_count = 0
		if (len(conf["autoswitch"]) > 2):
			name = conf["autoswitch"][1]
		elif ( not( name in page_list ) ):
			name = page_list[0]
		loadPage(widget, name)
		setShadow(widget)


def widgetClicked(widget, x, y, button):
    pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass


#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
    pass


#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
	global page, as_state, mode
	try:
		imgPressed(widget, meter)
	except:
		pass
	name = page.__class__.__name__
	if ( name == "Network" ):
		if ( meter == page.module1.icon and button == 1 ):
			status = karamba.getTextValue(widget, page.module1.status)
			if ( status == "Disconnected" ):
				karamba.execute("sudo /sbin/ifup " + page.module1.iface)
				page.module1.toggleConnecting(widget)
			elif ( status == "Connected" ):
				karamba.execute("sudo /sbin/ifdown " + page.module1.iface)
				page.module1.toggleDisconnected(widget)
		if ( page.length == 6 ):
			if ( meter == page.module2.icon and button == 1 ):
				status = karamba.getTextValue(widget, page.module2.status)
				if ( status == "Disconnected" ):
					karamba.execute("sudo /sbin/ifup " + page.module2.iface)
					page.module2.toggleConnecting(widget)
				elif ( status == "Connected" ):
					karamba.execute("sudo /sbin/ifdown " + page.module2.iface)
					page.module2.toggleDisconnected(widget)
	
	if ( name == "Mail" ):
		for module in page.module_list:
			if ( meter == module.imgMail and button == 1 ):
				karamba.changeText(widget, module.text, "Updating")
				module.update(widget)
				
	if ( name == "Clock" ):
		if ( meter == page.text1 and button == 1 ):
			bool = page.mode
			if bool:
				page.mode, mode = False, False
			else:
				page.mode, mode = True, True
			karamba.writeConfigEntry(widget, "mode", str(mode))
	
	if ( name == "Calendar" ):
		if ( meter == page.previous and button == 1 ):
			if ( page.cal.month == 1 ):
				page.cal.month = 12
				page.cal.year = page.cal.year - 1
			else:
				page.cal.month = page.cal.month - 1
			page.cal.update(widget)
		if ( meter == page.next and button == 1 ):
			if ( page.cal.month == 12 ):
				page.cal.month = 1
				page.cal.year = page.cal.year + 1
			else:
				page.cal.month = page.cal.month + 1
			page.cal.update(widget)
		if ( meter == page.previous and button == 2 ):
			page.cal.year = page.cal.year - 1
			page.cal.update(widget)
		if ( meter == page.next and button == 2 ):
			page.cal.year = page.cal.year + 1
			page.cal.update(widget)
	
	if ( name == "Network" ):
		mod_list = [page.module1]
		if (page.length == 6 ):
			mod_list.append(page.module2)
		for module in mod_list:
			if ( meter in [module.ipText, module.trafficIn, module.trafficOut] and button == 1 ):
				if (module.page == 1):
					module.page = 2
					path = karamba.getThemePath(widget)
					karamba.setTextSensor(widget, module.trafficIn, 'sensor=program interval=5000 program="python ' + path + 'scripts/getIfaceTotal.py -d ' + module.iface + '"')
					karamba.setTextSensor(widget, module.trafficOut, 'sensor=program interval=5000 program="python ' + path + 'scripts/getIfaceTotal.py -u ' + module.iface + '"')
				elif (module.page == 2):
					module.page = 1
					karamba.setTextSensor(widget, module.trafficIn, 'sensor=network interval=1000 decimals=1 format="Download :\n%in kB/s" device="' + module.iface + '"')
					karamba.setTextSensor(widget, module.trafficOut, 'sensor=network interval=1000 decimals=1 format="Upload :\n%out kB/s" device="' + module.iface + '"')
				module.update(widget)
				karamba.redrawWidget(widget)
	
	if ( button == 1 ):
		if ( meter == infosImg ):
			page.close(widget)
			page = None
			loadPage(widget, 'SysInfos')
		if ( meter == cpuImg ):
			page.close(widget)
			page = None
			loadPage(widget, 'CpuMem')
		if ( meter == hddImg ):
			page.close(widget)
			page = None
			loadPage(widget, 'Partitions')
		if ( meter == netImg ):
			page.close(widget)
			page = None
			loadPage(widget, 'Network')
		if ( meter == clockImg ):
			page.close(widget)
			page = None
			loadPage(widget, 'Clock')
		if ( meter == calImg ):
			page.close(widget)
			page = None
			loadPage(widget, 'Calendar')
		if ( meter == mailImg ):
			page.close(widget)
			page = None
			loadPage(widget, 'Mail')
		if ( meter == battImg ):
			page.close(widget)
			page = None
			loadPage(widget, 'AcpiInfos')
	
	if ( meter == autoswitchImg and button == 1 ):
		if as_state:
			updateStateImg(widget, False)
		else:
			updateStateImg(widget, True)


#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
def itemDropped(widget, dropText):
    pass


#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.  
def startupAdded(widget, startup):
    pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.  
def startupRemoved(widget, startup):
    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.  
def taskRemoved(widget, task):
    pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).  
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    pass


# This will be printed when the widget loads.
print "Loaded Aero-AIO python extension!"
