import karamba
import os
import re
from time import strftime

class CalendarModule:
	
	def __init__(self, widget, theme, color, x, y):
		self.x = x
		self.y = y
		self.c_month = int(strftime("%m"))
		self.c_year = int(strftime("%Y"))
		self.month = self.c_month
		self.year = self.c_year
		
		self.text = karamba.createText(widget, x - 75, y, 300, 20, "")
		karamba.changeTextSize(widget, self.text, 16)
		karamba.setTextAlign(widget, self.text, "CENTER")
		karamba.changeTextColor(widget, self.text, color[0], color[1], color[2])
		
		self.bar = karamba.createImage(widget, x, y, "Themes/" + theme + "/calendar/bar.png")
		self.day = karamba.createImage(widget, x, y, "Themes/" + theme + "/calendar/day.png")
		self.column1 = karamba.createText(widget, x + 3, y + 24, 20, 12, "")
		self.column2 = karamba.createText(widget, x + 25, y + 24, 20, 12, "")
		self.column3 = karamba.createText(widget, x + 47, y + 24, 20, 12, "")
		self.column4 = karamba.createText(widget, x + 69, y + 24, 20, 12, "")
		self.column5 = karamba.createText(widget, x + 91, y + 24, 20, 12, "")
		self.column6 = karamba.createText(widget, x + 113, y + 24, 20, 12, "")
		self.column7 = karamba.createText(widget, x + 135, y + 24, 20, 12, "")
		
		for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
			karamba.changeTextColor(widget, meter, color[0], color[1], color[2])
	
	def displayCal(self, widget, month, year):
		self.month = month
		self.year = year
		fdate = str(month) + " " + str(year)
		
		f = os.popen('cal ' + fdate).readline()
		match = re.search("([\S]+ [0-9]+)", f)
		karamba.changeText(widget, self.text, match.group(1)[0].upper() + match.group(1)[1:])
		
		c1 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b -2").read()
		c2 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 4-5").read()
		c3 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 7-8").read()
		c4 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 10-11").read()
		c5 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 13-14").read()
		c6 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 16-17").read()
		c7 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 19-20").read()
		
		karamba.changeText(widget, self.column1, c1)
		karamba.changeText(widget, self.column2, c2)
		karamba.changeText(widget, self.column3, c3)
		karamba.changeText(widget, self.column4, c4)
		karamba.changeText(widget, self.column5, c5)
		karamba.changeText(widget, self.column6, c6)
		karamba.changeText(widget, self.column7, c7)
		
		for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
			karamba.changeTextFont(widget, meter, "monospace")
		
		if ( month == self.c_month and year == self.c_year ):
			found = False
			column = 1
			x = self.x
			y = self.y
			date = os.popen("date +%e").read()
			date = date.replace("\n", "")
			for c in [c1, c2, c3, c4, c5, c6, c7]:
				if ( found == False ):
					match = re.search("[a-z]+\n([0-9 ]+)\n([0-9 ]+)\n([0-9 ]+)\n([0-9 ]+)\n([0-9 ]+)?(\n([0-9 ]+))?", c)
					if ( date == match.group(1) ):
						found = True
						y = 13
					elif ( date == match.group(2) ):
						found = True
						y =26
					elif ( date == match.group(3) ):
						found = True
						y = 39
					elif ( date == match.group(4) ):
						found = True
						y = 52
					elif ( date == match.group(5) ):
						found = True
						y = 65
					elif ( date == match.group(7) ):
						found = True
						y = 78
					else :
						column = column + 1
			if ( column == 1):
				x = 0
			elif ( column == 2):
				x = 22
			elif ( column == 3):
				x = 44
			elif ( column == 4):
				x = 66
			elif ( column == 5):
				x = 88
			elif ( column == 6):
				x = 110
			elif ( column == 7):
				x = 132
			karamba.moveImage(widget, self.day, self.x + x, self.y + y + 24)
			karamba.moveImage(widget, self.bar, self.x + x, self.y + 25)
			karamba.showImage(widget, self.day)
			karamba.showImage(widget, self.bar)
		else :
			karamba.hideImage(widget, self.day)
			karamba.hideImage(widget, self.bar)
	
	def update(self, widget):
		self.displayCal(widget, self.month, self.year)
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.text, color[0], color[1], color[2])
		for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
			karamba.changeTextFont(widget, meter, "monospace")
	
	def shadow(self, widget, bool):
		if bool:
			karamba.changeTextShadow(widget, self.text, 1)
			for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
				karamba.changeTextShadow(widget, meter, 1)
		else:
			karamba.changeTextShadow(widget, self.text, 0)
			for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
				karamba.changeTextShadow(widget, meter, 0)
	
	def close(self, widget):
		karamba.deleteText(widget, self.text)
		for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
			karamba.deleteText(widget, meter)
		karamba.deleteImage(widget, self.bar)
		karamba.deleteImage(widget, self.day)

class Calendar:
	
	def __init__(self, widget, theme, color, titleColor):
		self.loaded = False
		self.pid = None
		self.count = 1200
		self.title = karamba.createText(widget, 36, 32, 200, 24, "Calendar")
		karamba.changeTextSize(widget, self.title, 20)
		karamba.changeTextColor(widget, self.title, titleColor[0], titleColor[1], titleColor[2])
		self.line = karamba.createImage(widget, 31, 58, "Themes/" + theme + "/line.png")
		
		self.next = karamba.createImage(widget, 277, 102, "Themes/" + theme + "/calendar/next.png")
		self.previous = karamba.createImage(widget, 51, 102, "Themes/" + theme + "/calendar/previous.png")
		karamba.attachClickArea(widget, self.next)
		karamba.attachClickArea(widget, self.previous)
		self.cal = CalendarModule(widget, theme, color, 105, 64)
		self.loaded = True
	
	def updateColor(self, widget, color):
		self.cal.updateColor(widget, color)
	
	def shadow(self, widget, bool):
		self.cal.shadow(widget, bool)
	
	def update(self, widget):
		if ( self.count < 1200 ):
			self.count = self.count + 1
		else:
			self.cal.update(widget)
			self.count = 1
	
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		karamba.deleteImage(widget, self.next)
		karamba.deleteImage(widget, self.previous)
		self.cal.close(widget)
