import karamba
import time
import math

class Hand:
	def __init__(self,widget,handimage,scale,pivot_x,pivot_y,center_x,center_y):
		self.imghand=karamba.createImage(widget,0,0,handimage)
		hw=karamba.getImageWidth(widget,self.imghand)
		hh=karamba.getImageHeight(widget,self.imghand)
		r1=math.sqrt((pivot_x*pivot_x)+(pivot_y*pivot_y))
		r4=r1
		a1=math.pi-math.asin(pivot_y/r1)
		a4=2.0*math.pi-a1
		r2=math.sqrt((pivot_y*pivot_y)+((hw-pivot_x)*(hw-pivot_x)))
		r3=r2
		a2=math.asin(pivot_y/r2)
		a3=-a2
		self.a1=a1
		self.a2=a2
		self.a3=a3
		self.a4=a4
		self.r1=r1*scale
		self.r2=r2*scale
		self.r3=r3*scale
		self.r4=r4*scale
		self.center_x=center_x
		self.center_y=center_y
		self.prev=0.0	#prevents rotating etc. if not required
		
	def draw(self,widget,angle):
		if (self.prev==angle):
			return
		self.prev=angle
		karamba.rotateImage(widget,self.imghand, int(angle))
		a=angle/180.0*math.pi
		x1=self.center_x+self.r1*math.cos(a+self.a1)
		y1=self.center_y+self.r1*math.sin(a+self.a1)
		x2=self.center_x+self.r2*math.cos(a+self.a2)
		y2=self.center_y+self.r2*math.sin(a+self.a2)
		x3=self.center_x+self.r3*math.cos(a+self.a3)
		y3=self.center_y+self.r3*math.sin(a+self.a3)
		x4=self.center_x+self.r4*math.cos(a+self.a4)
		y4=self.center_y+self.r4*math.sin(a+self.a4)
		xl=x1
		if (x2<xl):
			xl=x2
		if (x3<xl):
			xl=x3
		if (x4<xl):
			xl=x4
		yl=y1
		if (y2<yl):
			yl=y2
		if (y3<yl):
			yl=y3
		if (y4<yl):
			yl=y4
		xm=x1
		if (x2>xm):
			xm=x2
		if (x3>xm):
			xm=x3
		if (x4>xm):
			xm=x4
		ym=y1
		if (y2>ym):
			ym=y2
		if (y3>ym):
			ym=y3
		if (y4>ym):
			ym=y4
		karamba.resizeImage(widget,self.imghand,int(xm-xl),int(ym-yl))
		karamba.moveImage(widget,self.imghand,int(xl),int(yl))

class ClockModule:
	
	def __init__(self, widget, theme, color, x, y):
		self.bg = karamba.createImage(widget, x, y, "Themes/" + theme + "/clock/bg.png")
		self.houhand=Hand(widget,"Themes/" + theme + "/clock/hhand.png", 0.18, 0.0, 5.0, x+57, y+57)
		self.minhand=Hand(widget,"Themes/" + theme + "/clock/mhand.png", 0.25, 0.0, 5.0, x+57, y+57)
		self.sechand=Hand(widget, "Themes/" + theme + "/clock/shand.png", 0.27, 15.0, 5.0, x+57, y+57)
	
	def close(self, widget):
		karamba.deleteImage(widget, self.bg)
		karamba.deleteImage(widget, self.houhand.imghand)
		karamba.deleteImage(widget, self.minhand.imghand)
		karamba.deleteImage(widget, self.sechand.imghand)


class Clock:
	
	def __init__(self, widget, theme, color, titleColor, mode):
		self.loaded = False
		self.pid = None
		self.count = 600000
		self.interval = 600000
		self.mode = mode
		self.title = karamba.createText(widget, 36, 32, 200, 24, "Clock")
		karamba.changeTextSize(widget, self.title, 20)
		karamba.changeTextColor(widget, self.title, titleColor[0], titleColor[1], titleColor[2])
		self.line = karamba.createImage(widget, 31, 58, "Themes/" + theme + "/line.png")
		
		self.clock = ClockModule(widget, theme, color, 123, 63)
		self.text1 = karamba.createText(widget, 30, 101, 93, 12, "")
		self.text2 = karamba.createText(widget, 30, 113, 93, 12, "")
		karamba.attachClickArea(widget, self.text1)
		#karamba.setTextSensor(widget, self.text1, 'sensor=time format="hh:mm:ss" interval=1000')
		#karamba.setTextSensor(widget, self.text2, 'sensor=time format="dddd dd MMMM\nyyyy" interval=600000')
		karamba.setTextAlign(widget, self.text1, "CENTER")
		karamba.setTextAlign(widget, self.text2, "CENTER")
		self.uptime = karamba.createText(widget, 237, 101, 93, 12, "")
		karamba.setTextAlign(widget, self.uptime, "CENTER")
		karamba.setTextSensor(widget, self.uptime, 'sensor=uptime format="uptime\n%d day(s)\n%h h %m min" interval=60000')
		karamba.changeTextColor(widget, self.uptime, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.text1, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.text2, color[0], color[1], color[2])
		self.loaded = True
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.uptime, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.text1, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.text2, color[0], color[1], color[2])
	
	def shadow(self, widget, bool):
		if bool:
			karamba.changeTextShadow(widget, self.uptime, 1)
			karamba.changeTextShadow(widget, self.text1, 1)
			karamba.changeTextShadow(widget, self.text2, 1)
		else:
			karamba.changeTextShadow(widget, self.uptime, 0)
			karamba.changeTextShadow(widget, self.text1, 0)
			karamba.changeTextShadow(widget, self.text2, 0)
	
	def update(self, widget):
		if self.mode:
			karamba.changeText(widget, self.text1, time.strftime("%H:%M:%S"))
		else:
			karamba.changeText(widget, self.text1, time.strftime("%I:%M %p"))
		if ( self.count < self.interval ):
			self.count = self.count + 1
		else:
			self.count = 1
			karamba.changeText(widget, self.text2, time.strftime("%A %d %B\n%Y"))
		time_tuple = time.localtime()
		hours = time_tuple[3]
		minutes = time_tuple[4]
		seconds = time_tuple[5]
		self.clock.houhand.draw(widget,(hours-3)*30+int(minutes/5)*2.5)
		self.clock.minhand.draw(widget,(minutes-15)*6+int(seconds/15)*1.5)
		self.clock.sechand.draw (widget,(seconds-15)*6)
	
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		karamba.deleteText(widget, self.text1)
		karamba.deleteText(widget, self.text2)
		karamba.deleteText(widget, self.uptime)
		self.clock.close(widget)