import karamba
import os
import re

def cpuSpeed():
	try:
		speed = open("/sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq").read().replace("\n","")
	except:
		print "cpuSpeed : wrong file, trying 'proc/cpuinfo'..."
		speed = os.popen("cat /proc/cpuinfo | grep MHz | awk '{print $4}'").read().replace('\n','')
		speed = re.search("([0-9.]+)", speed)
		speed = speed.group(1)
		speed = int(float(speed))
		speed = (str(speed) + "MHz")
	else:
		speed = int(speed) / 1000
		speed = str(speed) + "Mhz"
	return speed

def cpuTemp(temp_file):
	if ( temp_file != "" ):
		temp = os.popen("cat " + temp_file + " | awk '{print $2}'").read().replace('\n','') + "C"
		return temp
	else:
		return "C"

class CpuModule:
	
	def __init__(self, widget, theme, color, graphColor, x, y):
		cpu_name = os.popen("echo `grep 'model name' /proc/cpuinfo| sed -n '/: /s/.............//p'`").read()
		self.name = karamba.createText(widget, x - 30, y, 200, 14, cpu_name)
		
		karamba.setTextAlign(widget, self.name, "CENTER")
		
		self.bar = karamba.createBar(widget, x + 2, y + 47, 136, 6, "Themes/" + theme + "/bar_140.png")
		karamba.setBarSensor(widget, self.bar, 'sensor=cpu format="%v" interval=1000')
		self.grid = karamba.createImage(widget, x, y + 15, "Themes/" + theme + "/cpu_grid.png")
		self.graph = karamba.createGraph(widget, x + 2, y + 17, 136, 26, 0)
		karamba.setGraphColor(widget, self.graph, graphColor[0], graphColor[1], graphColor[2])
		karamba.setGraphMinMax(widget, self.graph, 0, 100)
		karamba.setGraphSensor(widget, self.graph, 'sensor=cpu format="%v" interval=1000')
		self.load = karamba.createText(widget, x + 140, y + 24, 40, 12, "")
		karamba.setTextAlign(widget, self.load, "CENTER")
		karamba.setTextSensor(widget, self.load, 'sensor=CPU format="%v %" interval=1000')
		
		karamba.changeTextColor(widget, self.name, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.load, color[0], color[1], color[2])
		karamba.redrawWidget(widget)
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.name, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.load, color[0], color[1], color[2])
	
	def shadow(self, widget, bool):
		if bool:
			karamba.changeTextShadow(widget, self.name, 1)
			karamba.changeTextShadow(widget, self.load, 1)
		else:
			karamba.changeTextShadow(widget, self.name, 0)
			karamba.changeTextShadow(widget, self.load, 0)
	
	def close(self, widget):
		karamba.deleteText(widget, self.name)
		karamba.deleteText(widget, self.load)
		karamba.deleteImage(widget, self.grid)
		karamba.deleteBar(widget, self.bar)
		karamba.deleteGraph(widget, self.graph)

class MemModule:
	
	def __init__(self, widget, theme, color, x, y, mem_type, inter):
		self.count = inter
		self.interval = inter
		if ( mem_type == "s" ):
			mem_name = "Swap"
		elif ( mem_type == "m"):
			mem_name = "Ram"
		self.name = karamba.createText(widget, x, y, 100, 12, mem_name)
		self.percent = karamba.createText(widget, x + 100, y, 40, 12, "")
		self.used_total = karamba.createText(widget, x, y + 21, 140, 12, "")
		
		karamba.setTextAlign(widget, self.percent, "RIGHT")
		karamba.setTextAlign(widget, self.used_total, "CENTER")
		if ( mem_type == "m" ):
			karamba.setTextSensor(widget, self.used_total, 'sensor=memory format="%umb of %tm MB" interval=' + str(inter) + '000')
		else:
			karamba.setTextSensor(widget, self.used_total, 'sensor=memory format="%us of %ts MB" interval=' + str(inter) + '000')
		self.bar = karamba.createBar(widget, x + 2, y + 14, 136, 6, "Themes/" + theme + "/bar_140.png")
		self.border = karamba.createImage(widget, x, y + 12, "Themes/" + theme + "/border_140.png")
		karamba.setBarSensor(widget, self.bar, 'sensor=memory format="%u' + mem_type + 'b" interval=' + str(inter) + '000')
		
		karamba.changeTextColor(widget, self.name, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.percent, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.used_total, color[0], color[1], color[2])
	
	def update(self, widget):
		if ( self.count >= self.interval ):
			self.count = 1
			string = karamba.getTextValue(widget, self.used_total)
			string = string.replace('\n','')
			match = re.search("([0-9]+) of ([0-9]+) MB", string)
			if ( match != None ):
				used = float(match.group(1))
				total = float(match.group(2))
				perc = int((used / total)*100.0)
				karamba.changeText(widget, self.percent, str(perc) + "%")
		else:
			self.count = self.count + 1
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.name, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.percent, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.used_total, color[0], color[1], color[2])
	
	def shadow(self, widget, bool):
		if bool:
			karamba.changeTextShadow(widget, self.name, 1)
			karamba.changeTextShadow(widget, self.percent, 1)
			karamba.changeTextShadow(widget, self.used_total, 1)
		else:
			karamba.changeTextShadow(widget, self.name, 0)
			karamba.changeTextShadow(widget, self.percent, 0)
			karamba.changeTextShadow(widget, self.used_total, 0)
	
	def close(self, widget):
		karamba.deleteText(widget, self.name)
		karamba.deleteText(widget, self.percent)
		karamba.deleteText(widget, self.used_total)
		karamba.deleteImage(widget, self.border)
		karamba.deleteBar(widget, self.bar)


class CpuMem:
	
	def __init__(self, widget, theme, color, titleColor, graphColor, temp_file, interv):
		self.loaded = False
		self.pid = None
		self.count = interv
		self.interval = interv
		self.temp_file = temp_file
		self.title = karamba.createText(widget, 36, 32, 200, 24, "CPU and Memory")
		karamba.changeTextSize(widget, self.title, 20)
		karamba.changeTextColor(widget, self.title, titleColor[0], titleColor[1], titleColor[2])
		self.line = karamba.createImage(widget, 31, 58, "Themes/" + theme + "/line.png")
		
		self.freq = karamba.createText(widget, 40, 128, 130, 12, "")
		self.temp = karamba.createText(widget, 190, 128, 130, 12, "")
		self.moduleCpu = CpuModule(widget, theme, color, graphColor, 110, 65)
		self.moduleMem = MemModule(widget, theme, color, 37, 145, "m", interv)
		self.moduleSwap = MemModule(widget, theme, color, 183, 145, "s", interv)
		karamba.setTextAlign(widget, self.freq, "CENTER")
		karamba.setTextAlign(widget, self.temp, "CENTER")
		
		karamba.changeTextColor(widget, self.freq, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.temp, color[0], color[1], color[2])
		self.moduleCpu.updateColor(widget, color)
		self.moduleMem.updateColor(widget, color)
		self.moduleSwap.updateColor(widget, color)
		self.loaded = True
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.freq, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.temp, color[0], color[1], color[2])
		self.moduleCpu.updateColor(widget, color)
		self.moduleMem.updateColor(widget, color)
		self.moduleSwap.updateColor(widget, color)
	
	def shadow(self, widget, bool):
		if bool:
			karamba.changeTextShadow(widget, self.freq, 1)
			karamba.changeTextShadow(widget, self.temp, 1)
		else:
			karamba.changeTextShadow(widget, self.freq, 0)
			karamba.changeTextShadow(widget, self.temp, 0)
		self.moduleCpu.shadow(widget, bool)
		self.moduleMem.shadow(widget, bool)
		self.moduleSwap.shadow(widget, bool)
	
	def update(self, widget):
		if (self.count < self.interval):
			self.count = self.count + 1
		else:
			self.count = 0
			karamba.changeText(widget, self.freq, "CPU frequency : " + cpuSpeed())
			temp = cpuTemp(self.temp_file)
			if ( temp != "C" ):
				karamba.changeText(widget, self.temp, "CPU temp : " + temp)
				if ( karamba.getTextPos(widget, self.freq)[0] == 80 ):
					karamba.moveText(widget, self.freq, 40, 115)
			else:
				karamba.moveText(widget, self.freq, 110, 115)
		self.moduleMem.update(widget)
		self.moduleSwap.update(widget)
		
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		karamba.deleteText(widget, self.freq)
		karamba.deleteText(widget, self.temp)
		self.moduleCpu.close(widget)
		self.moduleMem.close(widget)
		self.moduleSwap.close(widget)
