import karamba

class MailModule:
	
	def __init__(self, widget, theme, color, x, y, name, server, username, password):
		self.serv = server
		self.user = str(username)
		self.passw = str(password)
		self.name = karamba.createText(widget, x, y + 8, 124, 18, name)
		self.imgMail = karamba.createImage(widget, x + 124, y, "Themes/" + theme + "/mail.png")
		self.num = karamba.createText(widget, x + 124, y + 8, 32, 18, "")
		self.text = karamba.createText(widget, x + 156, y + 8, 124, 18, "new message(s)")
		karamba.setTextAlign(widget, self.name, "CENTER")
		karamba.setTextAlign(widget, self.num, "CENTER")
		karamba.setTextAlign(widget, self.text, "CENTER")
		karamba.changeTextSize(widget, self.name, 14)
		karamba.changeTextSize(widget, self.num, 14)
		karamba.changeTextSize(widget, self.text, 14)
		karamba.attachClickArea(widget, self.imgMail)
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.name, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.num, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.text, color[0], color[1], color[2])
	
	def shadow(self, widget, bool):
		if bool:
			karamba.changeTextShadow(widget, self.name, 1)
			karamba.changeTextShadow(widget, self.num, 1)
			karamba.changeTextShadow(widget, self.text, 1)
		else:
			karamba.changeTextShadow(widget, self.name, 0)
			karamba.changeTextShadow(widget, self.num, 0)
			karamba.changeTextShadow(widget, self.text, 0)
	
	def update(self, widget):
		path = karamba.getThemePath(widget)
		if (self.serv == "gmail"):
			command = ["python", path + "scripts/getGMail.py", self.user, self.passw]
		else:
			command = ["python", path + "scripts/getNumMail.py", self.serv, self.user, self.passw]
		self.pid = karamba.executeInteractive(widget, command)
	
	def close(self, widget):
		karamba.deleteImage(widget, self.imgMail)
		karamba.deleteText(widget, self.name)
		karamba.deleteText(widget, self.num)
		karamba.deleteText(widget, self.text)

class Mail:
	
	def __init__(self, widget, theme, color, titleColor, account_list, inter):
		self.loaded = False
		self.interval = inter
		self.count = inter
		self.module_list = []
		self.pid = None
		self.title = karamba.createText(widget, 36, 32, 200, 24, "Mail")
		karamba.changeTextSize(widget, self.title, 20)
		karamba.changeTextColor(widget, self.title, titleColor[0], titleColor[1], titleColor[2])
		self.line = karamba.createImage(widget, 31, 58, "Themes/" + theme + "/line.png")
		
		number = len(account_list) / 4
		space = (129 - 32*number) / (number + 1)
		incr = space + 32
		y = 56 + space
		for i in range(number):
			self.module_list.append(MailModule(widget, theme, color, 40, y, account_list[ i * 4 ], account_list[ i * 4 + 1 ], account_list[ i * 4 + 2 ], account_list[ i * 4 + 3 ]))
			y = y + incr
			
		for module in self.module_list:
			module.updateColor(widget, color)
		self.loaded = True
	
	def updateColor(self, widget, color):
		for module in self.module_list:
			module.updateColor(widget, color)
	
	def shadow(self, widget, bool):
		for module in self.module_list:
			module.shadow(widget, bool)
	
	def update(self, widget):
		if ( self.count < self.interval ):
			self.count = self.count + 1
		else:
			self.count = 1
			for module in self.module_list:
				module.update(widget)
	
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		for module in self.module_list:
			module.close(widget)
