import karamba
import re
import os

class NetworkModule:
	
	def __init__(self, widget, theme, color, x, y, iface, max_in, max_out):
		self.theme = theme
		self.page = 1
		self.iface = iface
		iface_text = iface[0].upper() + iface[1:]
		self.ifaceText = karamba.createText(widget, x, y + 6, 100, 20, iface_text)
		karamba.changeTextSize(widget, self.ifaceText, 18)
		
		match = re.search("wlan", iface)
		if ( match != None ):
			self.icon = karamba.createImage(widget, x + 108, y, "Themes/" + theme + "/network/wireless.png")
		else:
			self.icon = karamba.createImage(widget, x + 108, y, "Themes/" + theme + "/network/network.png")
		karamba.attachClickArea(widget, self.icon)
		self.statusImg = karamba.createImage(widget, x + 131, y + 23, "")
		self.graphIn = karamba.createGraph(widget, x + 2, y + 34, 136, 26, 0)
		self.graphOut = karamba.createGraph(widget, x + 2, y + 34, 136, 26, 0)
		self.grid = karamba.createImage(widget, x, y + 32, "Themes/" + theme + "/bar_grid.png")
		karamba.setGraphColor(widget, self.graphIn, 0, 230, 0)
		karamba.setGraphColor(widget, self.graphOut, 230, 0, 0)
		karamba.setGraphMinMax(widget, self.graphIn, 0, max_in)
		karamba.setGraphMinMax(widget, self.graphOut, 0, max_out)
		karamba.setGraphSensor(widget, self.graphIn, 'sensor=network interval=1000 format="%in" device="' + iface + '"')
		karamba.setGraphSensor(widget, self.graphOut, 'sensor=network interval=1000 format="%out" device="' + iface + '"')
		
		self.status = karamba.createText(widget, x, y + 64, 140, 12, "")
		karamba.setTextAlign(widget, self.status, "CENTER")
		self.ipText = karamba.createText(widget, x, y + 78, 140, 12, "")
		karamba.setTextAlign(widget, self.ipText, "LEFT")
		self.trafficIn = karamba.createText(widget, x, y + 92, 70, 30, "")
		karamba.setTextSensor(widget, self.trafficIn, 'sensor=network interval=1000 decimals=1 format="Download :\n%in kB/s" device="' + iface + '"')
		self.trafficOut = karamba.createText(widget, x + 71, y + 92, 70, 30, "")
		karamba.setTextAlign(widget, self.trafficOut, "LEFT")
		karamba.setTextSensor(widget, self.trafficOut, 'sensor=network interval=1000 decimals=1 format="Upload :\n%out kB/s" device="' + iface + '"')
		for meter in [self.ipText, self.trafficIn, self.trafficOut]:
			karamba.attachClickArea(widget, meter)
		
		karamba.changeTextColor(widget, self.ifaceText, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.ipText, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.trafficIn, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.trafficOut, color[0], color[1], color[2])
	
	def toggleConnected(self, widget, ip):
		karamba.setImagePath(widget, self.statusImg, "Themes/" + self.theme + "/network/on.png")
		karamba.changeText(widget, self.status, "Connected")
		karamba.changeTextColor(widget, self.status, 0, 230, 0)
		if (self.page == 1):
			karamba.changeText(widget, self.ipText, "Ip : " + ip)
		elif (self.page == 2):
			karamba.changeText(widget, self.ipText, "External Ip : " + ip)
		karamba.showText(widget, self.ipText)
		karamba.showText(widget, self.trafficIn)
		karamba.showText(widget, self.trafficOut)
		
	def toggleConnecting(self, widget):
		karamba.setImagePath(widget, self.statusImg, "")
		karamba.changeText(widget, self.status, "Connecting...")
		karamba.changeTextColor(widget, self.status, 255, 150, 0)
	
	def toggleDisconnected(self, widget):
		karamba.setImagePath(widget, self.statusImg, "Themes/" + self.theme + "/network/off.png")
		karamba.changeText(widget, self.status, "Disconnected")
		karamba.changeTextColor(widget, self.status, 230, 0, 0)
		karamba.hideText(widget, self.ipText)
		karamba.hideText(widget, self.trafficIn)
		karamba.hideText(widget, self.trafficOut)
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.ifaceText, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.ipText, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.trafficIn, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.trafficOut, color[0], color[1], color[2])
	
	def shadow(self, widget, bool):
		if bool:
			karamba.changeTextShadow(widget, self.ifaceText, 1)
			karamba.changeTextShadow(widget, self.ipText, 1)
			karamba.changeTextShadow(widget, self.trafficIn, 1)
			karamba.changeTextShadow(widget, self.trafficOut, 1)
		else:
			karamba.changeTextShadow(widget, self.ifaceText, 0)
			karamba.changeTextShadow(widget, self.ipText, 0)
			karamba.changeTextShadow(widget, self.trafficIn, 0)
			karamba.changeTextShadow(widget, self.trafficOut, 0)
	
	def update(self, widget):
		ip = karamba.getIp(widget, self.iface)
		state = karamba.getTextValue(widget, self.status)
		if ( ip == "Disconnected" and state != "Connecting..." ):
			self.toggleDisconnected(widget)
		if ( ip != "Disconnected" ):
			if (self.page == 2):
				path = karamba.getThemePath(widget)
				f = os.popen("python " + path + "scripts/getExternalIP.py")
				ip = f.read()[0:-1]
				f.close()
			self.toggleConnected(widget, ip)
	
	def close(self, widget):
		karamba.deleteText(widget, self.ifaceText)
		karamba.deleteText(widget, self.ipText)
		karamba.deleteText(widget, self.trafficIn)
		karamba.deleteText(widget, self.trafficOut)
		karamba.deleteText(widget, self.status)
		karamba.deleteImage(widget, self.icon)
		karamba.deleteImage(widget, self.statusImg)
		karamba.deleteImage(widget, self.grid)
		karamba.deleteGraph(widget, self.graphIn)
		karamba.deleteGraph(widget, self.graphOut)


class Network:
	
	def __init__(self, widget, theme, color, titleColor, iface_list, inter):
		self.loaded = False
		self.title = karamba.createText(widget, 36, 32, 200, 24, "Network")
		karamba.changeTextSize(widget, self.title, 20)
		karamba.changeTextColor(widget, self.title, titleColor[0], titleColor[1], titleColor[2])
		self.line = karamba.createImage(widget, 31, 58, "Themes/" + theme + "/line.png")
		
		self.interval = inter
		self.count = inter
		
		self.length = len(iface_list)
		if ( self.length == 3 ):
			self.module1 = NetworkModule(widget, theme, color, 110, 60, iface_list[0], iface_list[1], iface_list[2])
		elif ( self.length == 6 ):
			self.module1 = NetworkModule(widget, theme, color, 36, 60, iface_list[0], iface_list[1], iface_list[2])
			self.module2 = NetworkModule(widget, theme, color, 184, 60, iface_list[3], iface_list[4], iface_list[5])
			self.module2.updateColor(widget, color)
		self.module1.updateColor(widget, color)
		self.loaded = True
	
	def updateColor(self, widget, color):
		self.module1.updateColor(widget, color)
		if ( self.length == 6 ):
			self.module2.updateColor(widget, color)
	
	def shadow(self, widget, bool):
		self.module1.shadow(widget, bool)
		if ( self.length == 6 ):
			self.module2.shadow(widget, bool)
	
	def update(self, widget):
		status = karamba.getTextValue(widget, self.module1.status)
		if ( status == "Connecting..." or self.count >= self.interval ):
			self.module1.update(widget)
			if ( self.length == 3 ):
				self.count = 1
		if ( self.length == 6 ):
			status2 = karamba.getTextValue(widget, self.module2.status)
			if ( status2 == "Connecting..." or self.count >= self.interval ):
				self.module2.update(widget)
				self.count = 1
		self.count = self.count + 1
	
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		self.module1.close(widget)
		if ( self.length == 6 ):
			self.module2.close(widget)
